% Optical performance.m
% Austin Roorda, June 20, 1999 (with many updates)
% This program is used to generate plots of the 
% wavefront, PSF, PTF and MTF from a set of preset coefficients or a file of Zernike coefficients

% The ordering of the Zernike polynomials is based on OSA standard ordering;

% *******************************************************************************************
% ************ PART 1: SET UP THE PROGRAM WITH APPROPRIATE PARAMETERS ***********************
% *******************************************************************************************
close all; clear all;
global PARAMS;	%set as global so that PARAMS(6) can be changed in a subroutine

PARAMS(1) = 400; 	% size of pupil aperture field in pixels (this defines the resolution of the calculation)
%PARAMS 2,3 and 4 are set later in the program because they depend on how the aberrations are read in
PARAMS(5) = 0.550;	% imaging wavelength in microns
PARAMS(6) = 0;		% number of pixels over which PSF is calculated (do not adjust here set in Zernikephase subroutine)
PARAMS(7) = 20; % increase to enhance the display of the wavefront (doesn't affect calculation)

% *******************************************************************************************
% ************ PART 2: GET THE ZERNIKE COEFFICIENTS *****************************************
% *******************************************************************************************

choice = inputdlg('1: Preset coeffs 2:*.zer file','Choose 1 or 2');
choice = str2num(choice{1});%-- 8/13/09  9:35 AM --%

if (choice==1) %use preset coefficients   
   % Zernike coeffs
   % Preset all coefficients to 0 and edit each value as desired
   
   % tilt
   c(1)=0; 	c(2)=0.0;
   %defocus and astigmatism
   c(3)=0; 	c(4)=0;	c(5)=0;
   %coma like
   c(6)=0.0;	c(7)=0.00;	c(8)=0;	c(9)=0.0;	
   %spherical aberration like
   c(10)=0;	c(11)=0.0;	c(12)=0.0;	c(13)=0;	c(14)=0;	
   %higher order (each row is a new radial order)
   c(15)=0;	c(16)=0;	c(17)=0;	c(18)=0;	c(19)=0;	c(20)=0;	
   c(21)=0;	c(22)=0;	c(23)=0;	c(24)=0;	c(25)=0;	c(26)=0;	c(27)=0;	
   c(28)=0;	c(29)=0;	c(30)=0;	c(31)=0;	c(32)=0;	c(33)=0;	c(34)=0;	c(35)=0;	
   c(36)=0;	c(37)=0;	c(38)=0;	c(39)=0;	c(40)=0;	c(41)=0;	c(42)=0;	c(43)=0;	c(44)=0;	
   c(45)=0;	c(46)=0;	c(47)=0;	c(48)=0;	c(49)=0;	c(50)=0;	c(51)=0;	c(52)=0;	c(53)=0;	c(54)=0;	
   c(55)=0;	c(56)=0;	c(57)=0;	c(58)=0;	c(59)=0;	c(60)=0;	c(61)=0;	c(62)=0;	c(63)=0;	c(64)=0;	c(65)=0;
   
   PARAMS(2) = 5;   % size of pupil in mm for which PSF and MTF is to be calculated
   PARAMS(3) = 5;	% size of pupil in mm that Zernike coefficients define. NOTE: You can define the aberrations
					% for any pupil size and calculate their effects for any smaller aperture.
    % Because of the reciprocal relationship between the pupil function and its Fourier transform,
    % it helps to define a pupil that makes up only a small central region of the pupil aperture.
    % Otherwise the point spread function is too small.
    PARAMS(4) = 60;	param4orig = PARAMS(4); 	% size of pupil field in mm (use a large field to magnify the PSF)
    
    %boxsize = 145.6; %set parameter 4 to get a specific PSF image dimension;
    %PARAMS(4) = 60*PARAMS(1)*(180/3.1416)*PARAMS(5)*.001/boxsize;

elseif (choice ==2) % Read a list of Zernike coefficients
      
   [fname,pname] = uigetfile('*.zer','Open Coefficient File');
   fid = fopen([pname fname],'r');
   version = fgetl(fid);% fscanf(fid,'%s',1);
   instrument = fgetl(fid);% fscanf(fid,'%s',1);
   manuf = fgetl(fid);% fscanf(fid,'%s',1);
   oper = fgetl(fid);% fscanf(fid,'%s',1);
   pupoff = fgetl(fid);% fscanf(fid,'%s',1);
   geooff = fgetl(fid);% fscanf(fid,'%s',1);
   datatype = fgetl(fid);% fscanf(fid,'%s',1);
   Rfit = fscanf(fid,'%s %f\n',[1 2]);% fscanf(fid,'%s',1);
   Rfit=Rfit(length(Rfit));
  	Rmax = fgetl(fid);% fscanf(fid,'%s',1);
   waverms = fgetl(fid);% fscanf(fid,'%s',1);
   order = fgetl(fid);% fscanf(fid,'%s',1);
   strehl = fgetl(fid);% fscanf(fid,'%s',1);
   refent = fgetl(fid);% fscanf(fid,'%s',1);
   refcor = fgetl(fid);% fscanf(fid,'%s',1);
   resspec = fgetl(fid);% fscanf(fid,'%s',1);
   data = fgetl(fid);% fscanf(fid,'%s',1);
   c = fscanf(fid,'%i %i %g',[3 inf]); %read the first line
   fclose(fid);
   c=c(3,2:66); %ignore the piston term (check this carefully!!!!!)
   
   %set these parameter based on what is contained in the *.zer file
   PARAMS(2)=2*Rfit; %default setting is the pupil size that the Zernike coeffs define, PARAMS(3)
   PARAMS(3)=2*Rfit; 
   PARAMS(4)=60; param4orig = PARAMS(4); %automatically compute the field size
   fprintf('\nThe current *.zer filename is %s\n',fname);
end

PARAMS; % display the parameters to the screen

% *******************************************************************************************
% ************ PART 3: COMPUTE THE SPECTACLE CORRECTION *************************************
% This section will calculate the best spectacle correction for the eye, considering all terms
% *******************************************************************************************
   
spectacleOSA; % call the script spectacle.m
 
% *******************************************************************************************
% ************* PART 4: Calculate the RMS of the wave aberration*****************************
% In this section, you can calculate the RMS for all terms or any subest of terms. For example,
% to calculate the rms for defocus, simply caculate the sqrt of the sum of the squares for terms 
% 3,4 and 5. To calculate the RMS for 3rd order terms and higher, calculate the sqrt of the 
% sum of the squares for terms 6 - 66. 
% NOTE: If you change the values of any of the coefficients at this stage, those changes will
% be applied to all the next calculations. So, if you want to calculate the PSF and MTF for 
% high order aberrations only, then set all low order terms to zero at this point.
% *******************************************************************************************

% Remove the useless terms
c(1)=0;    % this term is tilt and does not have any effect on image quality
c(2)=0;	   % this term is tilt and does not have any effect on image quality

% remove other terms as desired

c(3)=0;    % remove astigmatism
c(5)=0;    % remove astigmatism
c(4) = 0;  % remove defocus

fprintf('Defocus\tPupil\tRMS\tStrehl\n');

%this is a better way to transfor the coefficients for different pupil
%sizes. The function is from Linda Lundstrom. This transform is more
%useful since you can remove tilt and defocus terms from the new pupil
%set of coefficients. 
%function C2=TransformC(C1,dia1, dia2,tx,ty,thetaR)
% TransformC returns transformed Zernike coefficient set, C2, from the original set, C1,
% both in standard ANSI order, with the pupil diameter in mm as the first term.
% dia2new pupil diameter [mm]
% tx, tyCartesian translation coordinates [mm]
% thetaRangle of rotation [degrees]
% Scaling and translation is performed first and then rotation.

corig=c;%write the original coefficients into a new variable

PupilSize = PARAMS(2); %change this if you want to analyze the wavefront for a new pupil size. You can also add this to the for loop below

for DefocusInDiopters = 0; % You can use this line to generate outputs for a range of defocus values. Just set a range here and disable the lines above and below.
    
    %PupilSize = 1;
    
    PARAMS(2)=PupilSize;
    
    if (PARAMS(2) < PARAMS(3))
        c = TransformC(corig,PARAMS(3),PARAMS(2),0,0,0);
        c(1:5) = 0; %set tilt, defocus and astigmatism to zero    
    else
        c=corig;
    end
    
    c(4)=(1e6/(4*sqrt(3)))*DefocusInDiopters*((PARAMS(2)/2000)^2); % convert DefocusInDiopters into a Zernike coefficient

    % calculate the RMS
    rms=sqrt(sum(c(1:65).^2)); % compute the sum of the sqaure of all the coefficients in the list that you define. 

    % print the result to the screen
    fprintf('%g\t%g\t%g\t',DefocusInDiopters, PupilSize, rms);

    % *******************************************************************************************
    % ************ PART 5: GENERATE AND DISPLAY THE WAVEFRONT ABERRATION ************************
    % *******************************************************************************************

    compWaveOSA; % call the script compWaveOSA.m

    % *******************************************************************************************
    % ************ PART 6: GENERATE AND DISPLAY THE POINT SPREAD FUNCTION ***********************
    % *******************************************************************************************
    PARAMS(4) = param4orig; %reset PARAMS(4) to its original value just in case it was changed in another script
    compPSFOSA; % call the script compPSFOSA.m

    % write some parameters to the screen
    fprintf('%g\n',max(max(PSF))); %print the strehl ratio to the screen

    %************** save an image of a letter E for convolving ******************
    %This generates a letter E tha tis approriately scaled for the PSF that
    %you just calculated. 
    
    LetterSize=2.5; %This is the lettersize in minutes of arc: examples --> 20/10=2.5; 20/20 = 5; 20/40 = 10 ...
    a=MakeAnE(PARAMS(1)*LetterSize*60/plotdimension,PARAMS(1)); %the image is saved in the images folder. 
    
    if(1)
        %Save the point spread function into a tif file and identify each file by the defocus
        if (choice==1)
            name=['images\preset_' num2str(DefocusInDiopters) '_' num2str(PupilSize) '_PSF.tif'];
   %         invname=['images\inv_preset_' num2str(DefocusInDiopters) '_' num2str(PupilSize) '_PSF.tif'];  
        elseif (choice==2)
            name=['images\' fname '_' num2str(DefocusInDiopters) '_' num2str(PupilSize) '_PSF.tif'];
  %          invname=['images\inv_' fname '_' num2str(DefocusInDiopters) '_' num2str(PupilSize) '_PSF.tif'];           
        end
        PSF=PSF/(max(max(PSF)));
 %       PSFinv = 1-PSF;
        imwrite(PSF,name,'tif');
 %       imwrite(PSFinv, invname, 'tif');
    end

    % *******************************************************************************************
    % ************ PART 7: GENERATE AND DISPLAY THE MODULATION TRANSFER FUNCTION ****************
    % *******************************************************************************************
    if(0)%if you set the argument to 1, then the entire set of commands with the if-end loop is executed. Its a way to select what parts of the conde to run when you run a program. 
        
        compMTF_FFTbased;
        %compMTFPTF;
       
        compMTF_radial; %compute radial average MTF
        compMTF_HV; % compute MTFs in Horizontal and vertical directions

        [sizeMTF junk] = size(radialMTF);

        
        %write the radial average MTF to a file

        if (choice==1)
            name=['images/preset_' num2str(DefocusInDiopters) '_MTF.txt'];
        elseif (choice==2)
            name=['images/' fname '_' num2str(DefocusInDiopters) '_' num2str(PARAMS(2)) '_MTF.txt'];
        end

        fid = fopen(name,'w');

        for (idx=1:sizeMTF)
            fprintf(fid,'%d\t%d\n',axisradMTF(idx),radialMTF(idx));
        end
        
        fclose(fid);
    end
    drawnow;
end
